
using namespace std;

class dosing
{
    public:
        dosing();
        void printDebug(short pinID);  // used to print info to terminal
        void compileDoseJson();  // compile data to be sent to front end
        void calculateDoses(short pinID);  // calculate number of doses and runtime
        void startupDosing();  // run at startup
        void ruleStartStirrer(short pinID, short interval);  // this is called from custom rules
        void startStirrer(short pinID, short source);  // this is called from rules and schedules and starts the dosing process
        void stopStirrer(short pinID);  // used to turn off stirrer before dosing
        void startDosing(short pinID);  // used to start dosing pump
        void stopDosing(short pinID);  // used to stop dosing pump
        void monitorDosingPumps();  // monitor the dosing pumps in main loop
        void manualDose(short pinID);  // manual dose
        void manualDoseCalibrate(short pinID);  // manual dose when calibrating
        void refillReservoir(short pinID);  // refill reservoir
        void saveDoseAmount(short pinID, short v);  // save dose amount
        void saveReservoirSize(short pinID, short v);  // save reservoir size
        void savePumpRate(short pinID, short v);  // save pump rate
        void saveStirrer(short pinID, short v);  // save stirrer status
        void saveCalibrationAmount(short pinID, short v);  // save calibration amount
        void saveDoseInterval(short pinID, short v);  // save minimum interval between doses
        void doseCalibrationDone(short pinID);  // update system that calibration is finished
    private:
        short debug = 0;  // used to print info to terminal
        short dosePinID[20] = {};  // unique pinID from pinsAvailable table
        short dosePin[20] = {};  // gpio pin number
        short doseDriverID[20] = {};  // driverID
        short doseAmount[20] = {};  // stores dose amount in ml
        short dosesRemaining[20] = {};  // tracks number of doses remaining
        short doseReservoirSize[20] = {};  // stores reservoir size in ml
        short doseReservoirRemain[20] = {};  // stores remaining space in reservoir in ml
        short dosePumpRate[20] = {};  // stores pump rate in ms/ml
        short doseStirrer[20] = {};  // stores if stirrer is enabled
        short doseCalibrateAmount[20] = {};  // tracks dose amount used for calibrating vessel
        unsigned int doseRunTime[20] = {};  // stores how long pump should run in millis
        short doseStepRunning[20] = {};  // tracks which step the system is in during the dosing process
        short doseCalibrateRunning[20] = {};  // tracks is calibration is running
        short doseInterval[20] = {};  // track minimum interval between doses
        short doseRan[20] = {0};  // track which stage the dosing pump is in so we can tell frontend
		unsigned long long* prevMillisDosing = new unsigned long long[20];  // tracks millis when pump started running
		unsigned long long* delayBetweenDosesMillis = new unsigned long long[20];  // track how long until next dose is allowed
        short totalDosers = 0;  // total dosing ports found
        short tempDoseInterval = 0;  // track if a pump was started from custom rule
};
dosing::dosing(){}

void dosing::compileDoseJson() {  // compile data to be sent to front end
    for(short a=0; a < totalDosers; a++) {  // get pinID and status of each dosing port
        outgoingDataString += "{\"id\":\"";
        outgoingDataString += to_string(dosePinID[a]);
        outgoingDataString += "\",\"v\":\"";
        
        outgoingDataString += to_string(doseStepRunning[a]);
        outgoingDataString += "\"}";
        if (a < totalDosers - 1) outgoingDataString += ",";  // add comma if another sensor is coming
    }
}

void dosing::printDebug(short pinID) {  // debug dosing pumps
    if (debug == 1) {
        short arrayLoc = 0;
        for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
            if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
        }
        std::cout << "pinID                = " << pinID << std::endl;
        
        std::cout << "doseAmount           = " << doseAmount[arrayLoc] << std::endl;
        std::cout << "dosesRemaining       = " << dosesRemaining[arrayLoc] << std::endl;
        std::cout << "doseReservoirSize    = " << doseReservoirSize[arrayLoc] << std::endl;
        std::cout << "doseReservoirRemain  = " << doseReservoirRemain[arrayLoc] << std::endl;
        std::cout << "dosePumpRate         = " << dosePumpRate[arrayLoc] << std::endl;
        std::cout << "doseStirrer          = " << doseStirrer[arrayLoc] << std::endl;
        std::cout << "doseRunTime          = " << doseRunTime[arrayLoc] << std::endl;
        std::cout << "doseCalibrateRunning = " << doseCalibrateRunning[arrayLoc] << std::endl; 
        std::cout << "" << std::endl;   
    }
}

void dosing::calculateDoses(short pinID) {  // calculate number of doses and runtime
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }

    doseRunTime[arrayLoc] = dosePumpRate[arrayLoc] * doseAmount[arrayLoc];
    dosesRemaining[arrayLoc] = (doseReservoirRemain[arrayLoc] / doseAmount[arrayLoc]);
}

void dosing::startupDosing() {  // run at startup
	if (startupComplete == 0) {
        std::cout << " " << std::endl;
        std::cout << "startupDosing------------------------------" << std::endl;
    }
    
    MYSQL_RES *res;
    MYSQL_ROW row;
    
    // search database for pins defined as a dosing port and enabled
    connectMysql();

    std::string query = "SELECT pinsAvailable.pinID, pinsAvailable.driverID, pinsAvailable.gpio,\
        dosingPumps.doseAmount, dosingPumps.reservoirRemain, dosingPumps.reservoirSize, dosingPumps.dosePumpRate, dosingPumps.doseStirrer, dosingPumps.calibrateAmount, dosingPumps.doseInterval\
        FROM pinsAvailable\
        LEFT JOIN dosingPumps\
        ON pinsAvailable.pinID = dosingPumps.dosePinID\
        WHERE pinsAvailable.pinTypeID = 12 AND pinsAvailable.assigned = 1\
        ORDER BY dosingPumps.doseID ASC";

    mysql_query(conn, query.c_str());  // run query

    res = mysql_use_result(conn);
    std::fill_n(dosePinID, 20, 0);  // reset array
    std::fill_n(doseDriverID, 20, 0);  // reset array
    std::fill_n(dosePin, 20, 0);  // reset array
    std::fill_n(doseAmount, 20, 0);  // reset array
    std::fill_n(doseReservoirRemain, 20, 0);  // reset array
    std::fill_n(doseReservoirSize, 20, 0);  // reset array
    std::fill_n(dosePumpRate, 20, 0);  // reset array
    std::fill_n(doseStirrer, 20, 0);  // reset array
    std::fill_n(doseCalibrateAmount, 20, 0);  // reset array
    std::fill_n(doseRunTime, 20, 0);  // reset array
    std::fill_n(doseCalibrateRunning, 20, 0);  // reset array
    std::fill_n(doseInterval, 20, 0);  // reset array

    totalDosers = 0;
    while ((row = mysql_fetch_row(res)) != NULL)
    {
        dosePinID[totalDosers] = atoi(row[0]);
        doseDriverID[totalDosers] = atoi(row[1]);
        dosePin[totalDosers] = atoi(row[2]);
        doseAmount[totalDosers] = atoi(row[3]);
        doseReservoirRemain[totalDosers] = atoi(row[4]);
        doseReservoirSize[totalDosers] = atoi(row[5]);
        dosePumpRate[totalDosers] = atoi(row[6]);
        doseStirrer[totalDosers] = atoi(row[7]);
        doseCalibrateAmount[totalDosers] = atoi(row[8]);
        doseInterval[totalDosers] = atoi(row[9]);
        delayBetweenDosesMillis[totalDosers] = ((doseInterval[totalDosers] * 60) * 1000) + currentMillisTime;  // no dosing until dose interval has passed


        if (doseDriverID[totalDosers] == 2) {  // if dosing port is attached to Pi set pin as an output
	        pinMode(dosePin[totalDosers],OUTPUT);
        }

        totalDosers++;  // count number of dosing pumps
        calculateDoses(dosePinID[totalDosers-1]);
    }
    mysql_disconnect(); 
    mysql_free_result(res);
    


	if (startupComplete == 0) {
        std::cout << "dosePinID------------ ";
        for(short a=0; a < totalDosers; a++) {std::cout << dosePinID[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseDriverID--------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseDriverID[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "dosePin-------------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << dosePin[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseAmount----------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseAmount[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseReservoirRemain-- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseReservoirRemain[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseReservoirSize---- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseReservoirSize[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "dosePumpRate--------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << dosePumpRate[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseStirrer---------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseStirrer[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseCalibrateAmount-- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseCalibrateAmount[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseRunTime---------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseRunTime[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "dosesRemaining------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << dosesRemaining[a] << ", ";}
        std::cout << "" << std::endl;
        
        std::cout << "doseInterval--------- ";
        for(short a=0; a < totalDosers; a++) {std::cout << doseInterval[a] << ", ";}
        std::cout << "" << std::endl;

        std::cout << "totalDosers------------ " << totalDosers << std::endl;
        std::cout << "----------------------------------------" << std::endl;
    }
}

void dosing::ruleStartStirrer(short pinID, short interval) {  // this is called from custom rules
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }

    short source = 2;  // custom rule
    if (interval != doseInterval[arrayLoc]) {tempDoseInterval = interval;}  // if interval was added in rule capture in temporary variable to use set interval
    else {source = 1;}  // if no interval was added change source to schedule so we can use default minimum interval between doses
    startStirrer(pinID,source);  // start dosing process
}

void dosing::startStirrer(short pinID, short source) {  // this starts the dosing process
    // SOURCE
    //    0 - manual dose
    //    1 - schedule
    //    2 - custom rule

    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    
    if ((source == 0) || (currentMillisTime >= delayBetweenDosesMillis[arrayLoc]))  // if switching delay has passed or manual dose
    {
        if ((doseStirrer[arrayLoc] == 1) && (dosesRemaining[arrayLoc] > 0))  // if stirrer is enabled
        {
            doseStepRunning[arrayLoc] = 1;  // stirrer running
            prevMillisDosing[arrayLoc] = currentMillisTime;
        }
        else if ((doseStirrer[arrayLoc] == 0) && (dosesRemaining[arrayLoc] > 0)) {startDosing(pinID);}  // if no stirrer

        if (source == 2) {delayBetweenDosesMillis[arrayLoc] = ((tempDoseInterval * 60) * 1000) + currentMillisTime;}  // if dose was started from custom rule reset minimum interval between doses
        else if (source == 1) {delayBetweenDosesMillis[arrayLoc] = ((doseInterval[arrayLoc] * 60) * 1000) + currentMillisTime;}  // if not manual dose reset minimum interval between doses
    }
}

void dosing::stopStirrer(short pinID) {  // used to turn off stirrer before dosing
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "stopStirrer------------------------------" << arrayLoc << std::endl;

    doseStepRunning[arrayLoc] = 2;  // stirrer settling
    prevMillisDosing[arrayLoc] = currentMillisTime;
}

void dosing::startDosing(short pinID) {  // used to start dosing pump
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "startDosing------------------------------" << arrayLoc << std::endl;

    if (doseCalibrateRunning[arrayLoc] == 0) {  // update reservoir remaining if not calibrating
        doseReservoirRemain[arrayLoc] = doseReservoirRemain[arrayLoc] - doseAmount[arrayLoc];  // update reservior
        calculateDoses(dosePinID[arrayLoc]);
        // std::cout << "Save Dosing------------------------------" << arrayLoc << std::endl;

        connectMysql();
        std::string query = "UPDATE dosingPumps SET reservoirRemain = " + to_string(doseReservoirRemain[arrayLoc]) + " WHERE dosePinID = '" + to_string(dosePinID[arrayLoc]) + "'";  // update database that sensor is unplugged
        mysql_query(conn, query.c_str());  // run query
        mysql_disconnect();
    }
    
    if (doseDriverID[arrayLoc] == 2) {digitalWrite(dosePin[arrayLoc],1);}  // driver is Raspberry Pi
    else {pca9685.adjustPinLevel(pinID, 4095);}  // other pca9685 driver

    prevMillisDosing[arrayLoc] = currentMillisTime;
    doseStepRunning[arrayLoc] = 3;  // pump running
}

void dosing::stopDosing(short pinID) {  // used to stop dosing pump
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "stopDosing------------------------------" << arrayLoc << std::endl;

    doseStepRunning[arrayLoc] = 0;  // dosing finsihed
    
    if (doseDriverID[arrayLoc] == 2) {digitalWrite(dosePin[arrayLoc],0);}  // driver is Raspberry Pi
    else {pca9685.adjustPinLevel(pinID, 0);}  // other pca9685 driver
}

void dosing::monitorDosingPumps() {  // monitor the dosing pumps in main loop
    for (short a=0; a < totalDosers; a++)  // monitor each dosing pump
    {
        if ((doseStepRunning[a] == 1) && (currentMillisTime - prevMillisDosing[a] > 10000)) {stopStirrer(dosePinID[a]);}  // stop stirrer
        else if ((doseStepRunning[a] == 2) && (currentMillisTime - prevMillisDosing[a] > 12000)) {startDosing(dosePinID[a]);}  // start pump
        else if ((doseStepRunning[a] == 3) && (currentMillisTime - prevMillisDosing[a] > doseRunTime[a])) {stopDosing(dosePinID[a]);}  // stop pump
    }
}

void dosing::manualDose(short pinID) {  // manual dose
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "manualDose = " << pinID << std::endl;
    
    systemSettings.logActivity(1,3,dosePinID[arrayLoc],1);  // log action
    startStirrer(pinID, 0);
    printDebug(pinID);
}

void dosing::manualDoseCalibrate(short pinID) {  // manual dose when calibrating
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "manualDoseCalibrate = " << arrayLoc << std::endl;

    doseCalibrateRunning[arrayLoc] = 1;  // mark pump as being calibrated
    doseRunTime[arrayLoc] = dosePumpRate[arrayLoc] * doseCalibrateAmount[arrayLoc];  // update runtime with calibration amount
    startDosing(pinID);
    printDebug(pinID);
}

void dosing::refillReservoir(short pinID) {  // refill reservoir
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "refillReservoir = " << arrayLoc << std::endl;

    doseReservoirRemain[arrayLoc] = doseReservoirSize[arrayLoc];  // refill
    calculateDoses(pinID);

    connectMysql();
    std::string query = "UPDATE dosingPumps SET reservoirRemain = '" + to_string(doseReservoirSize[arrayLoc]) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::saveDoseAmount(short pinID, short v) {  // save dose amount
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "saveDoseAmount = " << arrayLoc << std::endl;

    doseAmount[arrayLoc] = v;  // update dose amount
    calculateDoses(pinID);

    connectMysql();
    std::string query = "UPDATE dosingPumps SET doseAmount = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::saveReservoirSize(short pinID, short v) {  // save reservoir size
    printDebug(pinID);

    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "saveReservoirSize = " << arrayLoc << std::endl;

    doseReservoirSize[arrayLoc] = v;  // update
    calculateDoses(pinID);

    connectMysql();
    std::string query = "UPDATE dosingPumps SET reservoirSize = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::savePumpRate(short pinID, short v) {  // save pump rate
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "savePumpRate = " << arrayLoc << std::endl;

    dosePumpRate[arrayLoc] = v;  // update
    calculateDoses(pinID);

    connectMysql();
    std::string query = "UPDATE dosingPumps SET dosePumpRate = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::saveStirrer(short pinID, short v) {  // save stirrer status
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "saveStirrer = " << arrayLoc << std::endl;

    doseStirrer[arrayLoc] = v;  // update

    connectMysql();
    std::string query = "UPDATE dosingPumps SET doseStirrer = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::saveCalibrationAmount(short pinID, short v) {  // save calibration amount
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "saveCalibrationAmount = " << arrayLoc << std::endl;

    doseCalibrateAmount[arrayLoc] = v;  // update

    connectMysql();
    std::string query = "UPDATE dosingPumps SET calibrateAmount = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::saveDoseInterval(short pinID, short v) {  // save minimum interval between doses
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "saveDoseInterval = " << arrayLoc << std::endl;

    doseInterval[arrayLoc] = v;  // update
    delayBetweenDosesMillis[arrayLoc] = ((doseInterval[arrayLoc] * 60) * 1000) + currentMillisTime;  // if not manual dose reset minimum interval between doses

    connectMysql();
    std::string query = "UPDATE dosingPumps SET doseInterval = '" + to_string(v) + "' WHERE dosePinID = '" + to_string(pinID) + "'";
    mysql_query(conn, query.c_str()); // run query
    mysql_disconnect(); 
    printDebug(pinID);
}

void dosing::doseCalibrationDone(short pinID) {  // update system that calibration is finished
    printDebug(pinID);
    short arrayLoc = 0;
    for(short a=0; a < totalDosers; a++) {  // search through all dosing pumps to find match
        if (dosePinID[a] == pinID) {arrayLoc = a; break;}  // get array location
    }
    // std::cout << "doseCalibrationDone = " << arrayLoc << std::endl;

    doseCalibrateRunning[arrayLoc] = 0;  // calibration finsihed
    calculateDoses(pinID);
    printDebug(pinID);
}

dosing dosing;
